module Hickory
  module LogSubscribers
    class ActionMailer < ActiveSupport::LogSubscriber
      def process(event)
        metadata = {
          mailer: event.payload[:mailer],
          action: event.payload[:action]
        }

        logger.info('mailer-process', metadata)
      end

      def deliver(event)
        payload = event.payload
        metadata = {
          mailer:     payload[:mailer],
          message_id: payload[:message_id],
          subject:    payload[:subject],
          to:         obfuscate_emails(payload[:to]),
          from:       obfuscate_emails(payload[:from])
        }

        logger.info('mailer-deliver', metadata)
      end

      private

      def obfuscate_emails(emails)
        emails.map do |email|
          email.sub(/([^\s]+)@([^\s]+)/) { |_| "#{$1[0...4]}****@#{$2[0...3]}***" }
        end.join(', ')
      end
    end
  end
end