module Hickory
  module LogSubscribers
    class ActiveJob < ActiveSupport::LogSubscriber
      def enqueue(event)
        logger.info('job-enqueued', job_metadata(event))
      end

      def enqueue_retry(event)
        metadata = job_metadata(event)
        metadata.merge!({
          error: event.payload[:error],
          wait: event.payload[:wait]
        })

        logger.info('job-retry-enqueued', metadata)
      end

      def perform_start(event)
        logger.info('job-perform-start', job_metadata(event))
      end

      def perform(event)
        logger.info('job-perform', job_metadata(event))
      end

      def retry_stopped(event)
        metadata = job_metadata(event)
        metadata.merge!({
          error: event.payload[:error]
        })

        logger.info('job-retry-stopped', metadata)
      end

      def retry_discard(event)
        metadata = job_metadata(event)
        metadata.merge!({
          error: event.payload[:error]
        })

        logger.info('job-discard', metadata)
      end

      private

      def job_metadata(event)
        job = event.payload[:job]

        {
          job_id: job.job_id,
          queue: job.queue_name,
          priority: job.priority,
          executions: job.executions
        }
      end

    end
  end
end