require 'logger'

require 'active_support/logger_silence'
require 'active_support/logger_thread_safe_level'

require 'hickory/formatters/json'
require 'hickory/formatters/simple'

module Hickory
  class Logger < ::Logger
    include ActiveSupport::LoggerThreadSafeLevel
    include ActiveSupport::LoggerSilence

    def initialize(*args)
      super
      self.formatter = Formatters::Simple.new
    end

    SEVERITIY_LABELS = %w(TRACE DEBUG INFO WARN ERROR FATAL UNKNOWN)

    %i[trace debug info warn error fatal unknown].each do |level|

      define_method(level) do |msg = nil, meta={}, &block|
        message = { message: msg }

        message.merge!(meta) if meta.is_a?(Hash) && !meta.empty?

        severity = SEVERITIY_LABELS.index(level.to_s.upcase) - 1

        if block
          send(level, block.call)
        else
          log(severity, message)
        end
      end
    end

    delegate :push_tags, :pop_tags, :clear_tags!, to: :formatter

    def tagged(*tags)
      formatter.tagged(*tags) { yield self }
    end

    def flush
      clear_tags!
      super if defined?(super)
    end
  end
end